@@ -103,7 +103,7 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
         if "PLAYWRIGHT_DEFAULT_NAVIGATION_TIMEOUT" in settings:
             with suppress(TypeError, ValueError):
                 self.default_navigation_timeout = float(
-                    crawler.settings.get("PLAYWRIGHT_DEFAULT_NAVIGATION_TIMEOUT")
+                    settings.get("PLAYWRIGHT_DEFAULT_NAVIGATION_TIMEOUT")
                 )
 
         # headers
@@ -122,6 +122,7 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
             self.abort_request = load_object(settings["PLAYWRIGHT_ABORT_REQUEST"])
 
         self.context_rotation_idx = 0
+        self.max_timeouts_before_rotation: int = settings.getint("PLAYWRIGHT_TIMEOUTS_FOR_ROTATION") or 2
         self.timeout_errors_in_row = 0
         self.page_load_until: Literal["commit", "domcontentloaded", "load", "networkidle"] = settings.get(
             "PLAYWRIGHT_PAGE_LOAD_UNTIL") or "load"
@@ -148,7 +149,7 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
         logger.info(f"Browser {self.browser_type} launched")
 
     async def _plan_context_rotation(self) -> None:
-        logger.info("Re create browser: Before acquire")
+        logger.warning("Re create browser: Before acquire")
         await self.context_semaphore.acquire()
         logger.info("Re create browser: After acquire")
 
@@ -160,7 +161,7 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
 
         if self.context_semaphore.locked():
             self.context_semaphore.release()
-        logger.info(f"Re create browser: Completed with new context index {self.context_rotation_idx}")
+        logger.warning(f"Re create browser: Completed with new context index {self.context_rotation_idx}")
 
     async def _create_page(self) -> Page:
         """Create a new page in a context, also creating a new context if necessary."""
@@ -249,9 +250,6 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
         except PlaywrightTimeoutError:
             self.timeout_errors_in_row = self.timeout_errors_in_row + 1
             logger.error(f"[{request.url}] Request timeout while downloading request")
-            if self.timeout_errors_in_row > 2:
-                self.timeout_errors_in_row = 0
-                await self._plan_context_rotation()
             return Response('', status=504)
         except:
             logger.exception(f"[{request.url}] Error while downloading request")
@@ -274,6 +272,9 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
                 else:
                     if self.context_semaphore.locked():
                         self.context_semaphore.release()
+                if self.timeout_errors_in_row > self.max_timeouts_before_rotation:
+                    self.timeout_errors_in_row = 0
+                    await self._plan_context_rotation()
             except:
                 logger.exception(f"[{request.url}] Error on page close")
                 raise
